/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.plugin.util;

/*
 * Adapted from org.apache.maven.repository.digest.DefaultDigester which was released under the following
 * license:
 * <p>
 * Copyright 2001-2005 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.codehaus.plexus.util.IOUtil;

/**
 * Create a digest for a file.
 */
public class Digester {

    public static final String ROLE = Digester.class.getName();

    public static final String SHA1 = "SHA-1";

    public static final String MD5 = "MD5";

    private static final int CHECKSUM_BUFFER_SIZE = 16384;

    private static final int BYTE_MASK = 0xFF;

    // modified method with static qualifier TWW
    public static String createChecksum(File file, String algorithm) throws IOException, NoSuchAlgorithmException {
        MessageDigest digest = MessageDigest.getInstance(algorithm);

        InputStream fis = new FileInputStream(file);
        try {
            byte[] buffer = new byte[CHECKSUM_BUFFER_SIZE];
            int numRead;
            do {
                numRead = fis.read(buffer);
                if (numRead > 0) {
                    digest.update(buffer, 0, numRead);
                }
            } while (numRead != -1);
        } finally {
            IOUtil.close(fis);
        }

        return byteArrayToHexStr(digest.digest());
    }

    // modified method with static qualifier TWW
    public static boolean verifyChecksum(File file, String checksum, String algorithm) throws NoSuchAlgorithmException, IOException {
        // Create checksum for jar file
        String sum = createChecksum(file, algorithm);
        return checksum.toUpperCase().equals(sum.toUpperCase());
    }

    /**
     * Convert an incoming array of bytes into a string that represents each of the bytes as
     * two hex characters.
     * 
     * @param data
     */
    private static String byteArrayToHexStr(byte[] data) {
        String output = "";

        for (int cnt = 0; cnt < data.length; cnt++) {
            // Deposit a byte into the 8 lsb of an int.
            int tempInt = data[cnt] & BYTE_MASK;

            // Get hex representation of the int as a string.
            String tempStr = Integer.toHexString(tempInt);

            // Append a leading 0 if necessary so that each hex string will contain 2
            // characters.
            if (tempStr.length() == 1) {
                tempStr = "0" + tempStr;
            }

            // Concatenate the two characters to the output string.
            output = output + tempStr;
        }

        return output.toUpperCase();
    }

}
